"""Store any additional test parameters to PARAMETERS dictionary.
When pool of test cases is used, PARAMETERS can be used to store
DUT type specific parameters"""

PARAMETERS = {"param1": "this is param1", "param2": "and this is param2"}
